:: protop.bat
::
:: Description : Launches the ProTop Real-Time (RT) interactive client.
::
:: Parameters  : %1: database
::                   This is either: 
::                   - a friendlyName defined in %PROTOP%\etc\dblist.cfg
::                   or
::                   - a fully-qualified database path (with or without the .db extension)
::                     - in this case, we don't look for a matching line item in 
::                       etc/dblist.cfg 
::
:: Examples    : protop s2k
::               protop d:\path\sports2000
::               protop d:\path\sports2000.db
::
:: Dependencies: %PROTOP%\bin\protopenv.bat
::               %PROTOP%\bin\portalenv.bat
::               %PROTOP%\bin\dbaenv.bat
::               %PROTOP%\bin\localenv.bat
::
:: Note on code comments:
:: I prefer ":: " over "rem " for code comments but unfortunately they are not equivalent.
:: "Rem" statements are never interpreted. "::" comments *usually* aren't, with the exception
:: being complex statements within parentheses. That's why there's a mix of commenting styles
:: in this file.
:: -----------------------------------------------------------------------------------

@echo off

:: protop.bat requires a parameter
if "%1"=="" (

    echo.
    echo Error: Missing parameter
    echo        You must provide a friendly name that is defined in %PROTOP%\etc\dblist.cfg
    echo        or a fully-qualified database path, with or without the .db extension.
    echo E.g. : protop s2k
    echo        protop d:\path\sports2000
    echo        protop d:\path\sports2000.db
    echo.
    
    exit /b 1

)

:: Initialize variables and temporary file(s)

:: ISFILE   : set to "true" if %1 or %1.db is a file
:: XDB      : the fully-qualified database path; either from the command line or read from etc\dblist.cfg
:: FNAME    : the logical dbname from a fully-qualified database path, or the friendly name of the database in etc\dblist.cfg
:: XTYPE    : adds "-pf etc\%XTYPE%.pf" to PFLIST; read from "type" in etc\dblist.cfg (note that "type" is optional in dblist.cfg)
:: PFLIST   : " -pf <pfname1>.pf [-pf <pfname2>.pf ...]" (PFLIST may also be assigned values in protopenv.bat, which is called below)

set "ISFILE=false"
set "XDB="   
set "FNAME=" 
set "XTYPE=" 
set "PFLIST="

if exist %TMPDIR%\pt3dblist.tmp (

  del %TMPDIR%\pt3dblist.tmp
  
)

:: Is %1 a fully-qualified database path? (we're actually just testing that it's a file, and assuming that file is a valid database control area)
if exist "%1"    set ISFILE=true
if exist "%1".db set ISFILE=true

call "#ENVPT3#\bin\protopenv.bat"
 
:: The following command causes variables to be replaced with their literal values as each line is interpreted.
:: This overrides the default cmd.exe behaviour of replacing (or "expanding") variables at the start of each 
:: code block enclosed in parentheses. 
:: Note that within a code block, obtaining the most recently-set value of a variable also requires it to be 
:: referenced as !var! rather than %var%.
:: The SetLocal command also prevents variable changes within the batch file from persisting in the shell 
:: instance after the batch file completes, which is desirable.  Additionally, it prevents variables values set 
:: in batch files that are CALLed from persisting in the caller.  This would prevent protopenv.bat from having
:: any effect on the environment of protop.bat, which is why this command is issued after that call and not at
:: the beginning of this batch file.
SetLocal EnableDelayedExpansion

cd /d %PROTOP%

if "%ISFILE%"=="true" ( 

  rem    Notes on batch file "parameter extensions":
  rem      %~n1   is the filename (without extension) of the file specified by parameter %1
  rem      %~dpn1 is the drive, path, and filename of the file specified by parameter %1
  set "FNAME=%~n1"
  set "XDB=%~dpn1"
  rem    We don't set XTYPE in this case

  ) else if exist %PROTOP%\etc\dblist.cfg (

      rem    In this case, %1 must be a friendly name; does it have a line item in the config file?
      
      rem    Search for a line item in dblist.cfg that begins with the specified friendly name and a pipe
      rem    Note: as with ProTop on Unix, this is a case-sensitive search
      findstr /b "%1|" %PROTOP%\etc\dblist.cfg > %TMPDIR%\pt3dblist.tmp

      rem    If we found a line item in dblist.cfg with this friendly name, get XDB and XTYPE from it
      if !errorlevel!==0 (
        
        rem    In this case, the for /f statement parses the contents of a file: the temporary file
        rem    output by the findstr command above.  
        rem    It should always be a single-line file, as a friendly name should be unique in dblist.cfg.
        rem    If more than one line item exists in dblist.cfg for a given friendly name, the last one will
        rem    be used to determine the values of XDB and XTYPE.  
        rem 
        rem    Generically, a dblist.cfg line item looks like this (the pipes are token delimiters):
        rem 
        rem      friendlyName|/dbpath/dbName|serverName|monitor[|dlc[|type]]
        rem 
        rem    e.g.: s2k|d:\path\sports2000|myserver|yes 
        rem 
        rem    Notes:
        rem    - XDB, the fully-qualified database path, is token 2
        rem    - XTYPE, the "type" value, is token 6
        rem    - the first four tokens are mandatory
        rem    - "dlc" is optional
        rem    - if "dlc" is specified, "type" is optional
        rem    - if "type" is specified, "dlc" is mandatory
        rem 
        rem    The string after "for /f" is a list of options (explained below). "delims", if specified, must be last option. 
        rem      tokens=2,6 means: 
        rem      - "tokenize the line with the delimiter(s) and extract tokens 2 and 6, assigning their values to FOR parameters"
        rem      - "assign the first token value (token 2) to the FOR parameter specified after the options string (%%G)"
        rem      - "assign the next token value (token 6) to the next FOR parameter in sequence (in this case, %%H)" 
        rem      delims=|   means "use the pipe symbol as a delimiter to tokenize the line"
        
        for /f "tokens=2,6 delims=|" %%G in (%TMPDIR%\pt3dblist.tmp) do (
        
          set "XDB=%%G"
          set "XTYPE=%%H" 
          
        ) 

        set "FNAME=%1"

      rem    %1 wasn't a valid path, and dblist.cfg exists, but we did not find a matching line item for the friendly name  
      ) else ( 

          echo.
          echo Error: Friendly name not found in configuration file
          echo        You must provide a friendly name that is defined in %PROTOP%\etc\dblist.cfg
          echo        or a fully-qualified database path, with or without the .db extension.
          echo E.g. : protop s2k
          echo        protop d:\path\sports2000
          echo        protop d:\path\sports2000.db
          echo.
          
          exit /b 2
      
      )
    
      del %TMPDIR%\pt3dblist.tmp
      
  rem    %1 wasn't a valid path and %PROTOP\etc\dblist.cfg does not exist
  ) else (

      echo.
      echo Error: Configuration file does not exist
      echo        You must provide a friendly name that is defined in %PROTOP%\etc\dblist.cfg
      echo        or a fully-qualified database path, with or without the .db extension.
      echo        A sample configuration can be found in %PROTOP%\estub\dblist.cfg.
      echo E.g. : protop s2k
      echo        protop d:\path\sports2000
      echo        protop d:\path\sports2000.db
      echo.
      
      exit /b 3
    
)

:: Chain together .pf files if they exist:
::
::   etc/protop.pf       - moved to protopenv.bat
::   etc/custId.pf       - moved to protopenv.bat
::   etc/serverName.pf   - moved to protopenv.bat
::   etc/type.pf  
::   etc/friendlyName.pf

if exist %PROTOP%\etc\!XTYPE!.pf (

  set "PFLIST=%PFLIST% -pf etc\!XTYPE!.pf"

)

if exist %PROTOP%\etc\!FNAME!.pf (

  set "PFLIST=%PFLIST% -pf etc\!FNAME!.pf"

)

:: We now have the information we need to run ProTop RT
%DLC%\bin\_progres.exe -p protop.p -param %FNAME% -basekey "INI" -ininame etc\protop.ini %PFLIST% -T %TMPDIR% %XDB% %ERRSTACK% -clientlog %TMPDIR%\protop.%FNAME%.debug
